@@ -10,10 +10,11 @@ MARSSoptim = function(MLEobj) {
       cat(tmp)
       stop("Stopped in MARSSoptim() because marssMLE object is incomplete or inconsistent.\n", call.=FALSE)
     }
-#  for(elem in c("Q","R")){
-#    if(dim(MLEobj$model$free[[elem]])[3]>1)
-#      stop(paste("Stopped in MARSSoptim() because function does not allow free$",elem," to be time-varying.\n",sep=""), call.=FALSE)      
-#  }
+  for(elem in c("Q","R")){
+    if(dim(MLEobj$model$free[[elem]])[3]>1)
+     stop(paste("Stopped in MARSSoptim() because function does not allow estimated part of ",elem," to be time-varying.\n",sep=""), call.=FALSE)      
+  }
+  #the is.marssMODEL code has an additional test that the blocks are diagonal or unconstrained in the varcov matrices
 
 #  if(MLEobj$model$diffuse)
 #      stop(paste("Stopped in MARSSoptim() because version 3.0 does not allow diffuse=TRUE.\n",sep=""), call.=FALSE)      
@@ -43,31 +44,35 @@ MARSSoptim = function(MLEobj) {
   if(control$trace==-1)control$trace=0
     
   #The code is used to set things up to use MARSSvectorizeparam to just select inits for the estimated parameters
+  #Q=t(chol(Q)%*%chol(Q)); t(chol(Q)) has 0 in upper triangle
   tmp.MLEobj = MLEobj
+  #This is needed for the likelihood calculation
+  tmp.MLEobj$fixed.original = tmp.MLEobj$marss$fixed
+  tmp.MLEobj$free.original = tmp.MLEobj$marss$free
+  
   tmp.MLEobj$par = tmp.inits  #set initial conditions for estimated parameters
   for(elem in c("Q","R","V0")){ #need the chol for these
-        f=sub3D(modelObj$fixed[[elem]],t=1) #initial conditions are based on t=1; arbitrary
-        d=sub3D(modelObj$free[[elem]],t=1) #free[[elem]] is required to be time constant
-        the.par=orig.par=unvec(f+d%*%tmp.inits[[elem]], dim=par.dims[[elem]][1:2])
-        is.zero=diag(orig.par)==0   #where the 0s on diagonal are
+        d=sub3D(free[[elem]],t=1) #free[[elem]] is required to be time constant
+        f=sub3D(fixed[[elem]],t=1)
+        the.par=unvec(f+d%*%tmp.inits[[elem]], dim=par.dims[[elem]][1:2])
+        is.zero=diag(the.par)==0   #where the 0s on diagonal are
         if(any(is.zero)) diag(the.par)[is.zero]=1    #so the chol doesn't fail if there are zeros on the diagonal
-        the.par=t(chol(the.par))  #transpose of chol
+        the.par=t(chol(the.par))  #convert to transpose of chol
         if(any(is.zero)) diag(the.par)[is.zero]=0  #set back to 0
         if(!is.fixed(free[[elem]])){
-          tmp.MLEobj$par[[elem]] = solve(crossprod(d))%*%t(d)%*%(vec(the.par)-f) #from f+Dm=M and if f!=0, D==0
+          #from f+Dm=M so m = solve(crossprod(d))%*%t(d)%*%(vec(the.par)-f)
+          #but if d!=0,then f==0. if f!-0, then d==0.  
+          #Thus crossprod(d))%*%t(d) has 0 cols where fs appear in the.par and f is not needed
+          tmp.MLEobj$par[[elem]] = solve(crossprod(d))%*%t(d)%*%vec(the.par) 
         }else{ tmp.MLEobj$par[[elem]] = matrix(0,0,1) }
-        #when being passed to optim, pars for var-cov mat is the chol, so need to reset free and fixed
-        #step 1, compute the D matrix corresponding to upper.tri=0 at in t(chol)
-        tmp.list.mat=fixed.free.to.formula(sub3D(tmp.MLEobj$marss$fixed[[elem]],t=1),sub3D(tmp.MLEobj$marss$free[[elem]],t=1),par.dims[[elem]][1:2])
+        #when being passed to optim, pars for var-cov mat is the chol, so need to reset free so we can get the L=t(chol) matrix
+        #we don't need to reset fixed because it won't be used;
+        #step 1, compute the D matrix corresponding to upper.tri=0 in t(chol)
+        #note this only works because it is required that 
+        #a) if f!=0, d=0. so 1+a never appears in var-cov mat  b) a+b never appears in a var-cov mat, c) for BFGS, var-cov mat is time-invariant
+        tmp.list.mat=fixed.free.to.formula(f,d, par.dims[[elem]][1:2])
         tmp.list.mat[upper.tri(tmp.list.mat)]=0   #set upper tri to zero
         tmp.MLEobj$marss$free[[elem]]=convert.model.mat(tmp.list.mat)$free
-        #step 2, set the fixed part to the t(chol); need to take the chol of fixed elements; upper tri will be set to 0
-        tmp.fixed=unvec(f,dim=par.dims[[elem]][1:2])  #by definition the estimated elements will have f=0 since this is a varcov mat
-        is.zero = diag(tmp.fixed)==0 #need to deal with zeros on diagonal
-        if(any(is.zero)) diag(tmp.fixed)[is.zero]=1    #so the chol doesn't fail if there are zeros on the diagonal
-        chol.fixed=t(chol(tmp.fixed)) #chol of the fixed part of var-cov matrix
-        if(any(is.zero)) diag(chol.fixed)[is.zero]=0  #reset back to zero   
-        tmp.MLEobj$marss$fixed[[elem]][,,1]=vec(chol.fixed) #above required that dim3 of fixed$Q and R is 1
     }
   # will return the inits only for the estimated parameters
   pars = MARSSvectorizeparam(tmp.MLEobj)
@@ -117,8 +122,8 @@ MARSSoptim = function(MLEobj) {
         }
     } #end for
     
-      pars = MARSSvectorizeparam(tmp.MLEobj)  #now the pars values are adjusted back to normal scaling
-      #now put the estimated values back into MLEobj; fixed values set by $fixed
+      pars = MARSSvectorizeparam(tmp.MLEobj)  #now the pars values have been adjusted back to normal scaling
+      #now put the estimated values back into the original MLEobj; fixed and free matrices as in original
       MLEobj.return = MARSSvectorizeparam(MLEobj.return, pars)
       kf.out = MARSSkf(MLEobj.return)
       }else{
@@ -161,28 +166,31 @@ MARSSoptim = function(MLEobj) {
 neglogLik = function(x, MLEobj=NULL){  #NULL assignment needed for optim call syntax
 #MLEobj is tmp.MLEobj so has altered free and fixed
 #x is the paramvector
+    
+    #update the MLEobj by putting the estimated pars from optim in
     MLEobj = MARSSvectorizeparam(MLEobj, x)
     free=MLEobj$marss$free
-    fixed=MLEobj$marss$fixed
     pars=MLEobj$par
     par.dims=attr(MLEobj[["marss"]],"model.dims")
-    m=par.dims[["x"]][1]
-    n=par.dims[["y"]][1]
   for(elem in c("Q","R","V0")){
      if(!is.fixed(free[[elem]])) #recompute par if needed since par in parlist is transformed
-        {        
-        d=sub3D(free[[elem]],t=1) #this will be the one with the upper tri zero-ed out but ok since symmetric
+        {
+        d=sub3D(free[[elem]],t=1) #this will be the one with the upper tri zero-ed out
         par.dim=par.dims[[elem]][1:2]
-        #t=1 since D not allowed to be time-varying?
-        L=unvec(free[[elem]][,,1]%*%pars[[elem]],dim=par.dim) #this by def will have 0 row/col at the fixed values
+        #t=1 since D not allowed to be time-varying; since code 4 lines down won't work otherwise
+        L=unvec(d%*%pars[[elem]],dim=par.dim) #this by def will have 0 row/col at the fixed values
         the.par = tcrossprod(L)#L%*%t(L)
         #from f+Dm=M and if f!=0, D==0 so can leave off f
         MLEobj$par[[elem]]=solve(crossprod(d))%*%t(d)%*%vec(the.par)
         #solve(t(d)%*%d)%*%t(d)%*%vec(the.par)
         }
     } #end for over elem
+    #This function is passed a special MLEobj with a marss.original element
+    MLEobj$marss$fixed = MLEobj$fixed.original
+    MLEobj$marss$free = MLEobj$free.original
+
     #kfsel selects the Kalman filter / smoother function based on MLEobj$fun.kf
     negLL = MARSSkf( MLEobj, only.logLik=TRUE, return.lag.one=FALSE )$logLik
     
     -1*negLL
-     }
+    }
